-------------------------------------------------------------------------------
--=> AlertFrame.lua
--> A popup frame for giving notices and alerts
-------------------------------------------------------------------------------
--> LuaCheck Settings
-- luacheck: no max line length
-- luacheck: ignore  _.* for k self
--> weizPVP Globals
-- luacheck: globals weizPVP
--> Addon Libs Globals
-- luacheck: globals LibStub ViragDevTool_AddData
--> WoW Globals
-- luacheck: globals CreateFrame UIParent PlaySoundFile C_Timer

-------------------------------------------------------------------------------
--=> GLOBALS TO LOCALS
-------------------------------------------------------------------------------
local weizPVP = weizPVP

-------------------------------------------------------------------------------
--=> LIBS
-------------------------------------------------------------------------------
local SM = LibStub:GetLibrary("LibSharedMedia-3.0")

-------------------------------------------------------------------------------
--=> LOCALS
-------------------------------------------------------------------------------
--> Get
local topOffset = UIParent:GetTop() / 3

-------------------------------------------------------------------------------
--=> GLOBALS
-------------------------------------------------------------------------------

-- environments/stars/brokenshore_sky_sargeras01.m2

--=> AlertFrame
-----------------------------------------------------------
weizPVP.AlertFrame = CreateFrame("Frame", "weizPVP.AlertFrame", UIParent)
weizPVP.AlertFrame:SetFrameStrata("HIGH")
weizPVP.AlertFrame:SetPoint("TOP", UIParent, "CENTER", 0, topOffset + 60)
weizPVP.AlertFrame:SetHeight(90)
weizPVP.AlertFrame:SetWidth(160)
weizPVP.AlertFrame:SetBackdrop(
    {
        bgFile = SM:Fetch("background", "weizPVP-Background"),
        tile = false,
        edgeFile = SM:Fetch("border", "weizPVP-Border"),
        edgeSize = 2,
        insets = {left = 0, right = 0, top = 0, bottom = 0}
    }
)
weizPVP.AlertFrame:SetBackdropColor(0, 0, 0, 0.6)
weizPVP.AlertFrame:SetBackdropBorderColor(0, 0, 0, 0.8)
weizPVP.AlertFrame:SetFrameLevel(2)
weizPVP.AlertFrame:Hide()

--=> ModelBG
-----------------------------------------------------------
weizPVP.Settings.alertFrameModelPath = "spells/saberon_incense_state_purple.m2"
weizPVP.AlertModelBG = CreateFrame("CinematicModel", "weizPVP_AlertModelBG", weizPVP.AlertFrame)
weizPVP.AlertModelBG:SetPoint("CENTER", weizPVP.AlertFrame, "CENTER")
weizPVP.AlertModelBG:SetWidth(weizPVP.AlertFrame:GetWidth() - 4)
weizPVP.AlertModelBG:SetHeight(weizPVP.AlertFrame:GetHeight() - 4)
weizPVP.AlertModelBG:SetCamera(0)
weizPVP.AlertModelBG:SetModel(weizPVP.Settings.alertFrameModelPath)
weizPVP.AlertModelBG:SetAlpha(0.8)
weizPVP.AlertModelBG:SetFrameLevel(1)

--=> AlertFrame.TextTop
-----------------------------------------------------------
weizPVP.AlertFrame.TextTop = weizPVP.AlertFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
weizPVP.AlertFrame.TextTop:SetPoint("TOP", weizPVP.AlertFrame, "TOP", 0, -8)
weizPVP.AlertFrame.TextTop:SetFont(SM:Fetch("font", "Accidental Presidency"), 26, "OUTLINE")
weizPVP.AlertFrame.TextTop:SetJustifyH("CENTER")
weizPVP.AlertFrame.TextTop:SetTextColor(1, 0.3, 0.9, 1)
weizPVP.AlertFrame.TextTop:SetText("STEALTH DETECTED")
weizPVP.AlertFrame.TextTop:Hide()

--=> AlertFrame.TextBottom
-----------------------------------------------------------
weizPVP.AlertFrame.TextBottom = weizPVP.AlertFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
weizPVP.AlertFrame.TextBottom:SetPoint("TOP", weizPVP.AlertFrame.TextTop, "BOTTOM", 0, -4)
weizPVP.AlertFrame.TextBottom:SetFont(SM:Fetch("font", "Roboto Condensed Bold"), 24, "OUTLINE")
weizPVP.AlertFrame.TextBottom:SetJustifyH("CENTER")
weizPVP.AlertFrame.TextBottom:SetTextColor(1, 1, 1, 1)
weizPVP.AlertFrame.TextBottom:SetText("PLAYERNAME")
weizPVP.AlertFrame.TextBottom:Hide()

--=> AlertFrame.Text
-----------------------------------------------------------
--@ textTop = EVENT DESC
--@ textBottom = INFO/PLAYER NAME
--@ event = 'stealth'
-----------------------------------------------------------
local playSound = true
local function EnableSound()
    playSound = true
end
--> TEST VIA CHAT: /run weizPVP:ShowAlert("STEALTH DETECTED", "Playername")
function weizPVP:ShowAlert(textTop, textBottom)
    if weizPVP.AlertFrame.anim then
        weizPVP.AlertFrame.anim:Stop()
    else
        weizPVP.AlertFrame.anim = weizPVP.AlertFrame:CreateAnimationGroup()
        weizPVP.AlertFrame.anim:SetScript(
            "OnFinished",
            function()
                weizPVP.AlertFrame:Hide()
            end
        )

        local fade1 = weizPVP.AlertFrame.anim:CreateAnimation("Alpha")
        fade1:SetDuration(0.2)
        fade1:SetFromAlpha(0)
        fade1:SetToAlpha(1)
        fade1:SetEndDelay(3) --> Set the number of seconds the animation delays after finishing.
        fade1:SetOrder(1)

        local fade2 = weizPVP.AlertFrame.anim:CreateAnimation("Alpha")
        fade2:SetDuration(0.4)
        fade2:SetFromAlpha(1)
        fade2:SetToAlpha(0)
        fade2:SetOrder(2)
    end

    weizPVP.AlertFrame.TextTop:SetText(textTop)
    weizPVP.AlertFrame.TextBottom:SetText(textBottom)

    --> Set Background to resize based on text boundries
    local frameWidth
    local frameHeight = weizPVP.AlertFrame:GetTop() - weizPVP.AlertFrame.TextBottom:GetBottom() + 8
    weizPVP.AlertFrame:SetHeight(frameHeight)

    if weizPVP.AlertFrame.TextTop:GetWidth() > weizPVP.AlertFrame.TextBottom:GetWidth() then
        frameWidth = weizPVP.AlertFrame.TextTop:GetWidth()
    else
        frameWidth = weizPVP.AlertFrame.TextBottom:GetWidth()
    end

    weizPVP.AlertFrame:SetWidth(frameWidth + 48)
    weizPVP.AlertModelBG:SetWidth(weizPVP.AlertFrame:GetWidth() + 4)
    weizPVP.AlertModelBG:SetHeight(weizPVP.AlertFrame:GetHeight() + 4)

    --> Set Model
    weizPVP.AlertModelBG:SetModel(weizPVP.Settings.alertFrameModelPath)
    --> Show Elements
    weizPVP.AlertFrame.TextTop:Show()
    weizPVP.AlertFrame.TextBottom:Show()
    weizPVP.AlertFrame:SetAlpha(0)
    weizPVP.AlertFrame:Show()
    weizPVP.AlertFrame.anim:Play()
    --> play stealth alert sound
    if playSound then
        PlaySoundFile("Interface\\AddOns\\weizPVP\\Sounds\\alart-1.ogg")
        playSound = false
        C_Timer.After(3, EnableSound)
    end
end
