-------------------------------------------------------------------------------
--=> Events.lua
-- weizPVP: Events
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
--> LuaCheck Settings
-- luacheck: no max line length
-- luacheck: ignore  _.* for k self
--!! DEBUG
-- luacheck: globals weizPVP_DEBUG_previousOutput ViragDevTool_AddData
--> weizPVP: Libs
-- luacheck: globals LibStub LibInspect ViragDevTool
--> weizPVP: Globals
-- luacheck: globals weizPVPEvents weizPVP weizPVP.Settings WolrdPVP_InspectCache weizPVPInspect
--> WoW API Globals
-- luacheck: globals InCombatLockdown GetUnitName UnitIsEnemy UnitIsPlayer UnitClass UnitRace UnitName
-- luacheck: globals UnitLevel GetGuildInfo UnitHealth UnitHealthMax GetInspectSpecialization GetSpecializationRoleByID GetSpecializationInfoByID
-- luacheck: globals truncate  CanInspect UnitGUID time floor GetInventoryItemLink GetInventorySlotInfo GetItemInfo GetTime
-- luacheck: globals UnitIsPVP UnitIsDeadOrGhost ClearInspectPlayer NotifyInspect GetRealmName UnitIsDead UnitIsGhost UnitReaction
-- luacheck: globals strsub strfind bit COMBATLOG_OBJECT_REACTION_HOSTILE C_NamePlate CombatLogGetCurrentEventInfo UnitExists
-- luacheck: globals COMBATLOG_OBJECT_TYPE_PLAYER CopyTable GetPlayerInfoByGUID GetZonePVPInfo
-------------------------------------------------------------------------------
--=> GLOBALS TO LOCALS
-------------------------------------------------------------------------------
local _G = _G
local GetTime = _G.GetTime
local UnitIsEnemy = _G.UnitIsEnemy
local UnitIsPlayer = _G.UnitIsPlayer
local UnitClass = _G.UnitClass
local GetUnitName = _G.GetUnitName
local UnitName = _G.UnitName
local UnitLevel = _G.UnitLevel
local GetGuildInfo = _G.GetGuildInfo
local UnitIsPVP = _G.UnitIsPVP
local UnitIsDeadOrGhost = _G.UnitIsDeadOrGhost
local UnitHealth = _G.UnitHealth
local UnitHealthMax = _G.UnitHealthMax
local GetPlayerInfoByGUID = _G.GetPlayerInfoByGUID
local weizPVP = _G.weizPVP

-------------------------------------------------------------------------------
--=> LIBS
-------------------------------------------------------------------------------
local LibCLH = LibStub("LibCombatLogHealth-1.0")

-------------------------------------------------------------------------------
--=> LOCALS
-------------------------------------------------------------------------------
local _ = nil

-------------------------------------------------------------------------------
--=> GLOBALS
-------------------------------------------------------------------------------
weizPVP.MaxUnitsToParse = 40

-------------------------------------------------------------------------------
--=> GLOBAL FUNCTIONS
-------------------------------------------------------------------------------

--=> Update Player DB
--> All new entries and updates to weizPVP.PlayerDB happen here.
-----------------------------------------------------------------------
local lastAddedTime = 0
function weizPVP:UpdatePlayerDB(unitData)
    local nameAndRealm = unitData.nameAndRealm
    local playerDB = weizPVP.PlayerDB[nameAndRealm]

    if nameAndRealm and unitData.name ~= "Unknown" then --> If we have a name and a timestamp; update data.
        if playerDB then --> Update Existing entry if it exists
            --> UPDATE DB

            playerDB.LastUpdated = GetTime()

            --> SET GUILD
            if playerDB.Guild and unitData.guild then
                if playerDB.Guild ~= unitData.guild then
                    playerDB.Guild = unitData.guild --> Update Guild if changed
                end
            end

            --> SET CLASS
            if not playerDB.Class and unitData.class then
                playerDB.Class = unitData.class
            end

            --> SET RACE
            if not playerDB.Race and unitData.race then
                playerDB.Race = unitData.race
            end
            -- playerDB[nameAndRealm].Class = unitData.class or playerDB[nameAndRealm].Class
            -- playerDB[nameAndRealm].Race = unitData.race or playerDB[nameAndRealm].Race

            -->  UPDAE ESTIMATED VALUES
            if playerDB.Estimated == true then --> Handle Estimations
                playerDB.Estimated = unitData.estimated

                if unitData.level and playerDB.Level then --> If we have an existing level and an updated value
                    if unitData.level > playerDB.Level then --> If new level is higher, use thats
                        playerDB.Level = unitData.level or playerDB.Level
                    end
                else
                    playerDB.Level = unitData.level or playerDB.Level
                end
            else --> UNIT DATA ALREAY ENTERED; CHECK FOR UPDATE IN LEVEL
                --> CHECK FOR LEVEL UPDATE
                if not playerDB.Level and unitData.level and playerDB.Level < unitData.level then
                    playerDB.Level = unitData.level
                end
            end
            --> UPDATE DB
            weizPVP.PlayerDB[nameAndRealm] = playerDB

            --> UPDATE CACHE
            local timestamp = GetTime()
            if lastAddedTime >= timestamp then
                timestamp = lastAddedTime + 0.001
            end
            lastAddedTime = timestamp

            if not weizPVP.PlayerActiveCache[nameAndRealm] then
                weizPVP.PlayerActiveCache[nameAndRealm] = {}

                weizPVP.PlayerActiveCache[nameAndRealm].TimeAdded = timestamp
            end
            weizPVP.PlayerActiveCache[nameAndRealm].LastUpdated = timestamp

            weizPVP.PlayerActiveCache[nameAndRealm].Estimated = playerDB.Estimated
            weizPVP.PlayerActiveCache[nameAndRealm].Class = playerDB.Class
            weizPVP.PlayerActiveCache[nameAndRealm].Race = playerDB.Race
            weizPVP.PlayerActiveCache[nameAndRealm].Guild = playerDB.Guild
            weizPVP.PlayerActiveCache[nameAndRealm].Level = playerDB.Level

            if unitData.healthPercent then --> HEALTH
                weizPVP.PlayerActiveCache[nameAndRealm].HealthPercent = unitData.healthPercent
            end

            if unitData.dead ~= nil then --> DEAD
                weizPVP.PlayerActiveCache[nameAndRealm].Dead = unitData.dead
            end

            if unitData.stealth ~= nil then --> STEALTH
                weizPVP.PlayerActiveCache[nameAndRealm].Stealth = unitData.stealth
            end

            --> UPDATE PLAYER LIST
            weizPVP:AddPlayerData(nameAndRealm, timestamp, unitData.dead)

            return true
        else --> Create new entry in DB (Player was not in DB)
            local timestamp = GetTime()
            if lastAddedTime >= timestamp then
                timestamp = lastAddedTime + 0.001
            end
            lastAddedTime = timestamp

            weizPVP.PlayerDB[nameAndRealm] = {
                ["Name"] = unitData.name,
                ["Estimated"] = unitData.estimated,
                ["Realm"] = unitData.realm,
                ["Guild"] = unitData.guild,
                ["Class"] = unitData.class,
                ["Race"] = unitData.race,
                ["Level"] = unitData.level,
                ["LastUpdated"] = timestamp
            }
            weizPVP.PlayerActiveCache[nameAndRealm] = {
                ["Name"] = unitData.name,
                ["Estimated"] = unitData.estimated,
                ["Realm"] = unitData.realm,
                ["Guild"] = unitData.guild,
                ["Class"] = unitData.class,
                ["Race"] = unitData.race,
                ["Level"] = unitData.level,
                ["LastUpdated"] = timestamp,
                ["TimeAdded"] = timestamp,
                ["HealthPercent"] = unitData.healthPercent or 100,
                ["Dead"] = unitData.dead
            }
            weizPVP:AddPlayerData(nameAndRealm, timestamp, unitData.dead)
            return true
        end
    end
    return false
end

--=> Parse Unit Ability
-- Take data from Combat Log and process the data to determine more info on the unit involved
-----------------------------------------------------------------------
--// DEPRECATED
-- function weizPVP:ParseUnitAbility(event, player, flags, spellId, spellName)
function weizPVP:ParseUnitAbility(event, player, spellName)
    if weizPVP.MaxUnitsToParse >= weizPVP.NearbyCount then
        local abilityType = strsub(event, 1, 5)
        if
            (player ~= nil and spellName ~= nil) and
                (abilityType == "SWING" or abilityType == "SPELL" or abilityType == "RANGE")
         then
            local name = player
            local class
            local race
            local specName
            local level = 0

            local playerDataOUT
            --> Check AbilityDB
            local ability = weizPVP.AbilityDB[spellName]

            --> Check SpecDB
            local specTable = weizPVP.SpecDB[spellName]
            if specTable then
                class = specTable[1]
                specName = specTable[2]
            end

            --> class
            if ability and ability.class then
                class = ability.class
            elseif specTable and specTable[1] then
                class = specTable[1]
            end
            --> level
            if ability and ability.level then
                level = ability.level
            end
            --> race
            if ability and ability.race then
                race = ability.race
            end

            --> specName
            if specTable and specTable[2] then
                specName = specTable[2]
            end

            playerDataOUT = {
                ["name"] = name,
                ["class"] = class,
                ["level"] = level,
                ["race"] = race,
                ["specName"] = specName,
                ["ability"] = spellName
            }

            --> RETURN DATA
            if playerDataOUT.class or playerDataOUT.level or playerDataOUT.race or playerDataOUT.specName then
                return true, playerDataOUT
            else
                return false, {nil, nil, nil, nil, nil, nil}
            end
        end
    else
        return false, {nil, nil, nil, nil, nil, nil}
    end
    --> no player name or spell given; we can't do shit
    return false, {nil, nil, nil, nil, nil, nil}
end

--=> GET UNIT DATA
-----------------------------------------------------------------------
--> Gets data for events with a UNITID, such as 'target' or 'nameplate'
function weizPVP:GetUnitData(unit)
    --> Confirm Unit is a player that is an enemy or attackable
    local unitEnemy = false
    if UnitIsEnemy("player", unit) and UnitIsPlayer(unit) then
        unitEnemy = true
    end
    if unitEnemy then
        --> GET NAME & REALM
        local nameAndRealm = GetUnitName(unit, true)
        local estimated = false
        local name, realm = UnitName(unit, true)
        if not realm then
            realm = weizPVP.CurrentRealm -- set to myrealm name if we got nil back from UnitName
        end

        --> Timestamp
        local timestamp = GetTime()

        --> GET CLASS
        local class = UnitClass(unit)
        class = string.upper(class)
        class = string.gsub(class, "%s", "")

        --> GET RACE
        local race = select(1, UnitRace(unit))

        --> GET LEVEL
        local level = tonumber(UnitLevel(unit))

        --> SET STEALTH
        local stealth = false

        --> GET GUILD
        local guild = GetGuildInfo(unit)

        --> GET PVP FLAGGED
        local pvp = UnitIsPVP(unit)

        --> GET DEAD OR GHOST
        local dead = UnitIsDeadOrGhost(unit)

        --> GET HEALTH
        local healthPercent = UnitHealth(unit) / UnitHealthMax(unit) * 100

        local dataOUT = {
            ["nameAndRealm"] = nameAndRealm,
            ["name"] = name,
            ["estimated"] = estimated,
            ["realm"] = realm,
            ["guild"] = guild,
            ["class"] = class,
            ["race"] = race,
            ["level"] = level,
            ["timestamp"] = timestamp,
            ["pvp"] = pvp,
            ["dead"] = dead,
            ["stealth"] = stealth,
            ["healthPercent"] = healthPercent
        }

        return dataOUT
    end

    return nil
end

-------------------------------------------------------------------------------
--=> EVENT FUNCTIONS
-------------------------------------------------------------------------------

--=> EVENT: Player Mouseover
-----------------------------------------------------------------------
function weizPVP:PlayerMouseoverEvent()
    --> MOUSEOVER UNIT
    if UnitIsPlayer("mouseover") then
        local unitData = weizPVP:GetUnitData("mouseover")

        if unitData then
            weizPVP:UpdatePlayerDB(unitData)
        end
    end
    --> MOUSEOVER |> TARGET UNIT
    if UnitIsPlayer("mouseovertarget") then
        local unitData = weizPVP:GetUnitData("mouseovertarget")
        if unitData then
            weizPVP:UpdatePlayerDB(unitData)
        end
    end
    --> MOUSEOVER |> TARGET |> TARGET UNIT
    if UnitIsPlayer("mouseovertargettarget") then
        local unitData = weizPVP:GetUnitData("mouseovertargettarget")
        if unitData then
            weizPVP:UpdatePlayerDB(unitData)
        end
    end
end

--=> EVENT: Player Target Changed
-----------------------------------------------------------------------
function weizPVP:PlayerTargetEvent()
    --> TARGET INFO
    if UnitIsPlayer("target") then
        local unitData = weizPVP:GetUnitData("target")
        if unitData then
            weizPVP:UpdatePlayerDB(unitData)
        end
    end
    --> TARGET OF TARGET INFO
    if UnitIsPlayer("targettarget") then
        local unitData = weizPVP:GetUnitData("targettarget")
        if unitData then
            weizPVP:UpdatePlayerDB(unitData)
        end
    end
    --> TARGET OF TARGET OF TARGET INFO
    if UnitIsPlayer("targettargettarget") then
        local unitData = weizPVP:GetUnitData("targettargettarget")
        if unitData then
            weizPVP:UpdatePlayerDB(unitData)
        end
    end
end

--=> EVENT: UNIT_HEALTH
-----------------------------------------------------------------------
function weizPVP:UnitHealth(_, unit, _)
    if UnitIsEnemy("player", unit) and UnitIsPlayer(unit) then
        local healthPercent = select(3, LibCLH.UnitHealth(unit))
        local nameAndRealm = GetUnitName(unit, true)
        local dead = UnitIsDeadOrGhost(unit)

        if weizPVP.PlayerActiveCache[nameAndRealm] and nameAndRealm and (healthPercent or dead) then
            --> Should be updating PlayerCache, not the DB for health updates
            weizPVP.PlayerActiveCache[nameAndRealm].HealthPercent = healthPercent
            weizPVP.PlayerActiveCache[nameAndRealm].Dead = dead
            weizPVP.PlayerActiveCache[nameAndRealm].LastUpdated = GetTime()

            --> UPDATE PLAYER LIST
            weizPVP:AddPlayerData(nameAndRealm, GetTime(), dead)
        else
            local unitData = weizPVP:GetUnitData(unit)
            if unitData then
                weizPVP:UpdatePlayerDB(unitData)
            end
        end
    end
end

--=> EVENT: CCOMBAT_LOG_UNFILTERED
-----------------------------------------------------------------------
function weizPVP:CombatLogEvent()
    local timestamp,
        event,
        _,
        srcGUID,
        srcName,
        srcFlags,
        _,
        dstGUID,
        dstName,
        dstFlags,
        _,
        _,
        arg2,
        _,
        _,
        _,
        _,
        _,
        _,
        _,
        _ = CombatLogGetCurrentEventInfo()

    --> Dont scan yourself
    if dstName == srcName and srcName == weizPVP.CurrentCharName then
        return
    end

    --> Source INFO
    local isPlayer = bit.band(srcFlags, COMBATLOG_OBJECT_TYPE_PLAYER) == COMBATLOG_OBJECT_TYPE_PLAYER
    local isHostile = bit.band(srcFlags, COMBATLOG_OBJECT_REACTION_HOSTILE) == COMBATLOG_OBJECT_REACTION_HOSTILE
    if isPlayer and isHostile and srcGUID and srcName then
        local srcType = strsub(srcGUID, 1, 6)

        --> The unit is a player
        if srcType == "Player" then
            --> Parse Ability and find class/race/level
            --> weizPVP:ParseUnitAbility(event, player, flags, spellId, spellName)
            if weizPVP.PlayerDB[srcName] and weizPVP.PlayerDB[srcName].Estimated == true then
                --// DEPRECATED
                -- local parsedAbility, parsedAbilityData = weizPVP:ParseUnitAbility(event, srcName, srcFlags, arg1, arg2)
                local parsedAbility, parsedAbilityData = weizPVP:ParseUnitAbility(event, srcName, arg2)
                if parsedAbility and parsedAbilityData then --> If we got data from parsing ability and player isnt in cache, add player to cache
                    local name = string.gsub(srcName, " %- ", "-")
                    local class = select(2, GetPlayerInfoByGUID(srcGUID))
                    local stealth
                    local zoneInfo = select(1, GetZonePVPInfo())
                    class = class or parsedAbilityData.class
                    if
                        event == "SPELL_AURA_APPLIED" and ((arg2 == "Stealth") or (arg2 == "Prowl")) and srcName and
                            (tostring(zoneInfo) ~= "sanctuary true")
                     then
                        stealth = true
                        weizPVP:ShowAlert("Stealth Detected!", srcName, {["r"] = 0, ["b"] = 0, ["g"] = 0, ["a"] = 0.8})
                    end
                    local unitData = {
                        ["nameAndRealm"] = srcName,
                        ["name"] = name,
                        ["estimated"] = true,
                        ["class"] = class,
                        ["stealth"] = stealth,
                        ["race"] = parsedAbilityData.race,
                        ["level"] = parsedAbilityData.level,
                        ["timestamp"] = timestamp
                    }

                    weizPVP:UpdatePlayerDB(unitData)
                end
            else --> EXISTING UNIT; UPDATE
                local name = string.gsub(srcName, " %- ", "-")
                local stealth
                local zoneInfo = select(1, GetZonePVPInfo())
                if
                    event == "SPELL_AURA_APPLIED" and (arg2 == "Stealth") or
                        (arg2 == "Prowl") and srcName and (tostring(zoneInfo) ~= "sanctuary true")
                 then
                    stealth = true
                    weizPVP:ShowAlert("Stealth Detected!", srcName, {["r"] = 0, ["b"] = 0, ["g"] = 0, ["a"] = 0.8})
                end
                local unitData = {
                    ["nameAndRealm"] = srcName,
                    ["name"] = name,
                    ["stealth"] = stealth,
                    ["estimated"] = true,
                    ["timestamp"] = timestamp
                }

                weizPVP:UpdatePlayerDB(unitData)
            end
        end
    end

    --> Destination
    isPlayer = bit.band(dstFlags, COMBATLOG_OBJECT_TYPE_PLAYER) == COMBATLOG_OBJECT_TYPE_PLAYER
    isHostile = bit.band(dstFlags, COMBATLOG_OBJECT_REACTION_HOSTILE) == COMBATLOG_OBJECT_REACTION_HOSTILE
    if isPlayer and isHostile and dstGUID and dstName then
        local dstType = strsub(dstGUID, 1, 6)

        if dstType == "Player" then
            --> Parse Ability and find class/race/level
            --> weizPVP:ParseUnitAbility(analyseSpell, event, player, flags, spellId, spellName)
            if weizPVP.PlayerDB[dstName] and weizPVP.PlayerDB[dstName].Estimated == true then --> ESITMATED UNIT; GET INFO AND ADD
                local parsedAbility, parsedAbilityData = weizPVP:ParseUnitAbility(event, srcName, arg2)
                if parsedAbility then --> If we got data from parsing ability, update PlayerCache
                    local name = string.gsub(dstName, " %- ", "-")
                    local class = select(2, GetPlayerInfoByGUID(dstGUID))
                    class = class or parsedAbilityData.class
                    local dead
                    if event == "UNIT_DIED" then
                        dead = true
                    end
                    local unitData = {
                        ["nameAndRealm"] = dstName,
                        ["name"] = name,
                        ["estimated"] = true,
                        ["class"] = class,
                        ["dead"] = dead,
                        ["stealth"] = false,
                        ["race"] = parsedAbilityData.race,
                        ["level"] = parsedAbilityData.level,
                        ["timestamp"] = timestamp
                    }

                    weizPVP:UpdatePlayerDB(unitData)
                end
            else --> KNOW UNIT: UPDATE
                local name = string.gsub(dstName, " %- ", "-")
                local dead
                if event == "UNIT_DIED" or event == "UNIT_DESTROYED" then
                    dead = true
                end
                local unitData = {
                    ["nameAndRealm"] = dstName,
                    ["name"] = name,
                    ["dead"] = dead,
                    ["estimated"] = true,
                    ["timestamp"] = timestamp
                }

                weizPVP:UpdatePlayerDB(unitData)
            end
        end
    end
end

--=> EVENT: NAMEPLATE_ADDED
-- ====================================================
function weizPVP:NameplateAdded(_, unit)
    local unitData = weizPVP:GetUnitData(unit)
    if unitData then
        weizPVP:UpdatePlayerDB(unitData)
    end
end

--=> EVENT: NAMEPLATE REMOVED
-- ====================================================
function weizPVP:NameplateRemoved(_, unit)
    local unitData = weizPVP:GetUnitData(unit)
    if unitData then
        weizPVP:UpdatePlayerDB(unitData)
    end
end

--=> EVENT: ENTERING COMBAT
-----------------------------------------------------------------------
function weizPVP:EnteringCombat()
    weizPVP:SetStatusBarMessage("|cFFFF0000 + IN COMBAT|r", false, 0)
end

--=> EVENT: LEAVING COMBAT
-----------------------------------------------------------------------
function weizPVP:LeavingCombat()
    weizPVP:SetStatusBarMessage("|cFF42f489 - OUT OF COMBAT|r", true, 3)
end

--=> EVENT: Left Combat
-----------------------------------------------------------------------
function weizPVP:LeftCombatEvent()
end

--=> EVENT: Player Died
-----------------------------------------------------------------------
function weizPVP:PlayerDeadEvent()
end

--=> EVENT: Channel Notice
-----------------------------------------------------------------------
function weizPVP:ChannelNoticeEvent()
end

--=> EVENT: Zone Changed
-----------------------------------------------------------------------
function weizPVP:ZoneChangedEvent()
    -- weizPVP:ClearListData()
    -- if ViragDevTool then
    --     ViragDevTool_AddData("ZoneChangedEvent", "ZoneChangedEvent")
    -- end
end

--=> EVENT: Zone Changed + New Area
-----------------------------------------------------------------------
function weizPVP:ZoneChangedNewAreaEvent()
    -- weizPVP:ClearListData()
    -- if ViragDevTool then
    --     ViragDevTool_AddData("ZoneChangedNewAreaEvent", "ZoneChangedNewAreaEvent")
    -- end
end

--=> EVENT: Player Entering World
-----------------------------------------------------------------------
function weizPVP:PlayerEnteringWorldEvent()
end

--=> EVENT: Player Levled Up
-----------------------------------------------------------------------
function weizPVP:PlayerLevelUp()
    --> Set Updated Player Level
    weizPVP.PlayerLevel = UnitLevel("player") -- player's level
end
