-------------------------------------------------------------------------------
--=> Lockon.lua
-- Highlight targeted player's nameplates
-------------------------------------------------------------------------------
--> LuaCheck Settings
-- luacheck: no max line length
-- luacheck: ignore  _.* for k self
--> weizPVP Globals
-- luacheck: globals weizPVP weizPVP.Settings
--> Addon Libs Globals
-- luacheck: globals LibStub
--> WoW Globals
-- luacheck: globals CreateFrame UIParent RAID_CLASS_COLORS InCombatLockdown UnitSelectionColor GetTime WorldFrame
-- luacheck: globals UnitClass UnitIsPlayer IsItemInRange C_NamePlate IsUsableItem UnitIsEnemy UnitIsUnit GetPhysicalScreenSize
-------------------------------------------------------------------------------
--=> LIBS
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--=> GLOBALS
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--=> FUNCTIONS
-------------------------------------------------------------------------------
weizPVP.PLUGIN_LOCKON_ENABLED = true

local lineAlpha = 0.2
local circleAlpaha = 0.5
local arrowAlpha = 1

local RAID_CLASS_COLORS_TABLE = {
    ["Hunter"] = {r = 0.67, g = 0.83, b = 0.45},
    ["Warlock"] = {r = 0.53, g = 0.53, b = 0.93},
    ["Priest"] = {r = 1.0, g = 1.0, b = 1.0},
    ["Paladin"] = {r = 0.96, g = 0.55, b = 0.73},
    ["Mage"] = {r = 0.25, g = 0.78, b = 0.92},
    ["Rogue"] = {r = 1.0, g = 0.96, b = 0.41},
    ["Druid"] = {r = 1.0, g = 0.49, b = 0.04},
    ["Shaman"] = {r = 0.0, g = 0.44, b = 0.87},
    ["Warrior"] = {r = 0.78, g = 0.61, b = 0.43},
    ["Death Knight"] = {r = 0.77, g = 0.12, b = 0.23},
    ["Monk"] = {r = 0.0, g = 1.00, b = 0.59},
    ["Demon Hunter"] = {r = 0.64, g = 0.19, b = 0.79}
}

--=> CREATE MAIN FRAME
-----------------------------------------------------------
weizPVP.LockonFrame = CreateFrame("frame", "weizPVP:LockonFrame", WorldFrame)
weizPVP.LockonFrame:Hide()
weizPVP.LockonFrame:SetFrameLevel(0)
weizPVP.LockonFrame:SetFrameStrata("BACKGROUND")
weizPVP.LockonFrame:SetPoint("CENTER")
weizPVP.LockonFrame:SetSize(64, 64)

--=> CREATE CIRCLE TEXTURE
-----------------------------------------------------------
local circle = weizPVP.LockonFrame:CreateTexture(nil, "BACKGROUND")
circle:SetTexture([[interface/addons/weizPVP/plugins/lockon/circle.tga]])
circle:SetAllPoints()
circle:SetBlendMode("ADD")
circle:SetAlpha(circleAlpaha)

--=> CREATE LINES
-----------------------------------------------------------
local leftLine = weizPVP.LockonFrame:CreateLine("BACKGROUND")
leftLine:SetTexture("Interface\\addons\\weizPVP\\Textures\\white16x16.tga")
leftLine:SetThickness(2)
leftLine:SetStartPoint("LEFT", weizPVP.LockonFrame, -2000, 0)
leftLine:SetEndPoint("LEFT", weizPVP.LockonFrame, 8, 0)
leftLine:SetVertexColor(1, 1, 1)
leftLine:SetAlpha(lineAlpha)
leftLine:SetBlendMode("ADD")

local rightLine = weizPVP.LockonFrame:CreateLine("BACKGROUND")
rightLine:SetTexture("Interface\\addons\\weizPVP\\Textures\\white16x16.tga")
rightLine:SetThickness(2)
rightLine:SetStartPoint("RIGHT", weizPVP.LockonFrame, 2000, 0)
rightLine:SetEndPoint("RIGHT", weizPVP.LockonFrame, -8, 0)
rightLine:SetVertexColor(1, 1, 1)
rightLine:SetAlpha(lineAlpha)
rightLine:SetBlendMode("ADD")

local topLine = weizPVP.LockonFrame:CreateLine("BACKGROUND")
topLine:SetTexture("Interface\\addons\\weizPVP\\Textures\\white16x16.tga")
topLine:SetThickness(2)
topLine:SetStartPoint("TOP", weizPVP.LockonFrame, 0, 2000)
topLine:SetEndPoint("TOP", weizPVP.LockonFrame, 0, -8)
topLine:SetVertexColor(1, 1, 1)
topLine:SetAlpha(lineAlpha)
topLine:SetBlendMode("ADD")

local bottomLine = weizPVP.LockonFrame:CreateLine("BACKGROUND")
bottomLine:SetTexture("Interface\\addons\\weizPVP\\Textures\\white16x16.tga")
bottomLine:SetThickness(2)
bottomLine:SetStartPoint("BOTTOM", weizPVP.LockonFrame, 0, -2000)
bottomLine:SetEndPoint("BOTTOM", weizPVP.LockonFrame, 0, 8)
bottomLine:SetVertexColor(1, 1, 1)
bottomLine:SetAlpha(lineAlpha)
bottomLine:SetBlendMode("ADD")

--=> ARROWS
-----------------------------------------------------------
local arrows = weizPVP.LockonFrame:CreateTexture(nil, "BACKGROUND")
arrows:SetTexture([[interface/addons/weizPVP/plugins/lockon/arrows.tga]])
arrows:SetVertexColor(1, 0, 0)
arrows:SetAlpha(arrowAlpha)
arrows:SetAllPoints()

--> ARROWS: Rotation Group
local ag = arrows:CreateAnimationGroup()
local rotation = ag:CreateAnimation("Rotation")
rotation:SetDegrees(-360)
rotation:SetDuration(5)
ag:SetLooping("REPEAT")
ag:Play()

--> ARROWS: Fade In Group
local group = arrows:CreateAnimationGroup()
group:SetToFinalAlpha(true)
local AAG1 = group:CreateAnimation("Alpha")
AAG1:SetFromAlpha(0)
AAG1:SetToAlpha(1)
AAG1:SetDuration(0.5)

--> ARROWS: Scale 2x?
local scale1 = group:CreateAnimation("Scale")
scale1:SetScale(2, 2)
scale1:SetDuration(0)

--> ARROWS: Scale 50%?
local scale50 = group:CreateAnimation("Scale")
scale50:SetScale(0.5, 0.5)
scale50:SetDuration(0.5)

--> ARROWS: Fade Out
local fadeOut = weizPVP.LockonFrame:CreateAnimationGroup()
fadeOut:SetToFinalAlpha(true)
local AAG2 = fadeOut:CreateAnimation("Alpha")
AAG2:SetFromAlpha(1)
AAG2:SetToAlpha(0)
AAG2:SetDuration(0.2)
fadeOut:SetScript(
    "OnFinished",
    function(self)
        weizPVP.LockonFrame:Hide()
    end
)

--=> ARROWS: Fade in 2
local fadeIn = weizPVP.LockonFrame:CreateAnimationGroup()
fadeIn:SetToFinalAlpha(true)
local AAG3 = fadeIn:CreateAnimation("Alpha")
AAG3:SetOrder(1)
AAG3:SetFromAlpha(0)
AAG3:SetToAlpha(1)
AAG3:SetDuration(0.2)

fadeOut:SetScript(
    "OnFinished",
    function(self)
        weizPVP.LockonFrame:Hide()
    end
)

--=> netOmatic icon
local netTarget = weizPVP.LockonFrame:CreateTexture(nil, "BACKGROUND")
netTarget:SetTexture([[interface/ICONS/INV_Weapon_Rifle_38]])
netTarget:SetSize(16, 16)
netTarget:SetAlpha(1)
netTarget:SetPoint("BOTTOMLEFT", rightLine, "TOPLEFT", 0, 8)
netTarget:SetAlpha(0)

local targetPlate

--=> SET COLOR
local function SetColor(r, g, b)
    circle:SetVertexColor(r, g, b)
    topLine:SetVertexColor(r, g, b)
    bottomLine:SetVertexColor(r, g, b)
    leftLine:SetVertexColor(r, g, b)
    rightLine:SetVertexColor(r, g, b)
end

--=> SCALE COORDS
-----------------------------------------------------------
local xFactor, yFactor = 1, 1 -- pixel perfect stuff, just try and prevent it from screwing up our lines
local function ScaleCoords(xPixel, yPixel, trueScale)
    local x, y = xPixel / xFactor, yPixel / yFactor
    x, y = x - x % 1, y - y % 1 -- floor
    return trueScale and (xPixel * xFactor) or (x * xFactor), trueScale and (xPixel * xFactor) or (y * yFactor)
end

--=> Focus the "target"'s nameplate'
-----------------------------------------------------------
local Moving = false
function weizPVP:FocusPlate(plate)
    fadeOut:Stop()
    if not weizPVP.LockonFrame:IsShown() then
        local x, y = plate:GetCenter()
        if x and y then
            local scale = plate:GetEffectiveScale()
            x, y = x * scale, y * scale
            local fScale = weizPVP.LockonFrame:GetScale()
            x, y = x / fScale, y / fScale
            weizPVP.LockonFrame:SetPoint("CENTER", nil, "BOTTOMLEFT", ScaleCoords(x, y))
        end
        fadeIn:Play()
    end

    weizPVP.LockonFrame:Show()
    group:Play()
    targetPlate = plate

    --> Set Class Colors
    local r, g, b
    if UnitIsPlayer("target") then
        local class = UnitClass("target")
        if class then
            local colors
            colors = RAID_CLASS_COLORS_TABLE[class]
            r, g, b = colors.r, colors.g, colors.b
        else
            r, g, b = UnitSelectionColor("target")
        end
    else
        r, g, b = UnitSelectionColor("target")
    end
    --> APPLY COLORS
    SetColor(r, g, b)
    Moving = GetTime()
end

--=> TARGET CHANGED
-----------------------------------------------------------
function weizPVP.LockonFrame:PLAYER_TARGET_CHANGED()
    local nameplate = C_NamePlate.GetNamePlateForUnit("target") --weizPVP.LockonFrame:GetPlateByGUID(targetGUID)
    --> Activate on enemy
    if nameplate and UnitIsEnemy("player", "target") and UnitIsPlayer("target") then
        --> Fade out if not an enemy or no target
        targetPlate = nameplate
        weizPVP:FocusPlate(nameplate)
    else
        fadeOut:Play()
        targetPlate = nil
    end
end
weizPVP.LockonFrame:RegisterEvent("PLAYER_TARGET_CHANGED")

--=> PLAYER ENTERING WORLD
-----------------------------------------------------------
function weizPVP.LockonFrame:PLAYER_ENTERING_WORLD()
    self:PLAYER_TARGET_CHANGED()
end
weizPVP.LockonFrame:RegisterEvent("PLAYER_ENTERING_WORLD")

--=> ON UPDATE
-----------------------------------------------------------
local UpdateToggle = true --> update every other frame to lighten cpu loads
weizPVP.LockonFrame:SetScript(
    "OnUpdate",
    function(self)
        if UpdateToggle == true then
            topLine:SetStartPoint("TOP", weizPVP.LockonFrame, 0, UIParent:GetTop() - weizPVP.LockonFrame:GetTop())
            --if Moving and GetTime() - Moving > 0.75 then Moving = false end -- snap to the target if it's been moving for a while
            local plate = targetPlate
            if plate then
                --> net icon
                local inRange = IsItemInRange("Net-o-Matic 5000", "target")
                local usable = IsUsableItem("Net-o-Matic 5000")
                local isFlying = weizPVP:isFlying("target")

                if inRange and isFlying and usable then
                    netTarget:SetAlpha(1)
                else
                    netTarget:SetAlpha(0)
                end

                if Moving then
                    local frame1, frame2 = weizPVP.LockonFrame, targetPlate
                    local x1, y1 = frame1:GetCenter()
                    x1, y1 = x1 * frame1:GetEffectiveScale(), y1 * frame1:GetEffectiveScale()
                    local x2, y2 = frame2:GetCenter()
                    x2, y2 = x2 * frame2:GetEffectiveScale(), y2 * frame2:GetEffectiveScale()
                    local delta1, delta2 = y2 - y1, x2 - x1
                    local distance = (delta2 ^ 2 + delta1 ^ 2) ^ 0.5

                    local timeLeft = Moving + 1 - GetTime()

                    if timeLeft > 0 and distance > 3 then
                        -- Move distance / timeLeft toward frame2
                        -- elapsed / timeLeft
                        local amountToMove = distance / timeLeft / 10
                        local ratio = amountToMove / distance
                        -- Move a point along a line in a given direction
                        local x = x1 + ratio * delta2
                        local y = y1 + ratio * delta1
                        frame1:ClearAllPoints()
                        frame1:SetPoint("CENTER", nil, "BOTTOMLEFT", ScaleCoords(x, y))
                    else
                        frame1:ClearAllPoints()
                        frame1:SetPoint("CENTER", nil, "BOTTOMLEFT", ScaleCoords(x2, y2))
                        Moving = false
                    end
                else
                    --weizPVP.LockonFrame:SetPoint('CENTER', plate)
                    local x, y = plate:GetCenter()
                    local scale = plate:GetEffectiveScale()
                    x, y = x * scale, y * scale --(y - plate:GetHeight()/2) * scale
                    local fScale = weizPVP.LockonFrame:GetScale()
                    x, y = x / fScale, y / fScale
                    weizPVP.LockonFrame:SetPoint("CENTER", nil, "BOTTOMLEFT", ScaleCoords(x, y))
                end
            end
            UpdateToggle = false
        else
            UpdateToggle = true
        end
    end
)

--=> DISPLAY SIZE CHANGED
-----------------------------------------------------------
function weizPVP.LockonFrame:DISPLAY_SIZE_CHANGED()
    local width, height = GetPhysicalScreenSize()
    xFactor, yFactor = (768 / width) * (width / height), 768 / height
end
function weizPVP.LockonFrame:PLAYER_LOGIN()
    weizPVP.LockonFrame:DISPLAY_SIZE_CHANGED()
end

weizPVP.LockonFrame:RegisterEvent("DISPLAY_SIZE_CHANGED")
weizPVP.LockonFrame:RegisterEvent("PLAYER_LOGIN")

--=> NAMEPLATE ADDED
-----------------------------------------------------------
function weizPVP.LockonFrame:NAME_PLATE_UNIT_ADDED(unit)
    local nameplate = C_NamePlate.GetNamePlateForUnit(unit)
    if nameplate and UnitIsUnit("target", unit) and UnitIsEnemy("player", "target") and UnitIsPlayer("target") then
        targetPlate = nameplate
        weizPVP:FocusPlate(nameplate)
    end
end
weizPVP.LockonFrame:RegisterEvent("NAME_PLATE_UNIT_ADDED")

--=> NAMEPLATE REMOVED
-----------------------------------------------------------
function weizPVP.LockonFrame:NAME_PLATE_UNIT_REMOVED(unit)
    -- local nameplate = C_NamePlate.GetNamePlateForUnit(unit)
    if UnitIsUnit("target", unit) and UnitIsPlayer("target") and UnitIsEnemy("player", "target") then
        targetPlate = nil
        fadeOut:Play()
    end
end
weizPVP.LockonFrame:RegisterEvent("NAME_PLATE_UNIT_REMOVED")

weizPVP.LockonFrame:SetScript(
    "OnEvent",
    function(self, event, ...)
        return self[event] and self[event](self, ...)
    end
)
